﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web;
using RestSharp;

namespace SignorityAPI
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {

            /*  test server & key
             *  login url      : http://test.signority.com/UI/
             *  user     : user01@signority.com
             *  password : signority
             *  apikey   : cf1b5433-f6da-4195-972e-b9c81fdcd3b7
             */

            /*  API SDK Demo Account */
            var apikey = "e20451e9-dd09-4f7d-8366-f32e27db0e4a";
            var baseUrl = "https://sign.signority.com/api/";
            var UIUrl = "https://sign.signority.com/UI/";
            var username = "demo-1@signority.com";
            var password = "S1gnority2000!";
            

            //var client = new Signority(apikey, "");
            // or use user name + password
            var client = new Signority(username, password);
            client.baseUrl = baseUrl;

            Console.WriteLine("Signority API Demo ...");
            // 1. get account detail
            var account = client.GetAccount();
            Console.WriteLine("Your account:" + account.email + "(" + account.firstname + " " + account.lastname + ")" + 
                    ", account id:" + account.id + 
                    ", status:" + account.status);

            // 2. get doc list
            var doc_list = client.GetDocumentList(1, 5);
            Console.WriteLine("");
            Console.WriteLine("You have " + doc_list.documentCount + " documents.");
            Console.WriteLine("");
            foreach (Documents doc in doc_list.documents)
            {
                Console.WriteLine("DocId: " + doc.id + ", " + doc.title + ", " + doc.GUID + "," + doc.signingStatus);
            }

            // 3. get all matadata of a specific doc
            if(doc_list.documentCount > 0 )
            {
                int doc_id = doc_list.documents[0].id;

                var docDetail = client.GetOneDocument(doc_id);
                Console.WriteLine("");
                Console.WriteLine("the detail of the first doc:" + doc_id);
                Console.WriteLine("Invitations:" + docDetail.document.invitations.Count);
                foreach (Invitaion i in docDetail.document.invitations)
                {
                    Console.WriteLine("id:" + i.id + ", recipient:" + i.recipientName + "," + i.recipientEmail + "; GUID: " + i.GUID);
                }
            }
            

            // 4. send out a signing request 
            var doc1 = new Documents();
            doc1.title = "C# SDK example " + DateTime.Now;
            doc1.senderName = "C# SDK";
            doc1.senderEmail = "james.kou@signority.com";
            doc1.documentType = Code.DOCUMENT_TYPE_DOCUMENT;
            doc1.signingStatus = Code.SIGNING_STATUS_IN_PROGRESS; //  Code.SIGNING_STATUS_DRAFT;

            
            // [optional] set callback url 
            var docSettings = new DocumentSetting();
            docSettings.postBackUrl = "";  // "[http://your_callback_url]"; 
            docSettings.expireDaysAfterSent = 3650;
            doc1.documentSetting = docSettings;
            

            //  add signers
            var invitaions = new List<Invitaion>();
                var invitation = new Invitaion();
                invitation.setSigner("SigningHost", "james.kou@signority.com", Code.INVITATION_TYPE_SIGNING_HOST);

            //  add hosted signers / in person signers
                var hostedSigners = new List<HostedSigners>();
                var hostedSigners1 = new HostedSigners();
                hostedSigners1.setHostedSigner("signer 1", "sginer01@signority.com");
                var hostedSigners2 = new HostedSigners();
                hostedSigners2.setHostedSigner("signer 2", "sginer02@signority.com");
                hostedSigners.Add(hostedSigners1);
                hostedSigners.Add(hostedSigners2);
            invitation.hostedSigners = hostedSigners;
            invitaions.Add(invitation);

                //var invitaion2 = new Invitaion();
                //invitaion2.setSigner("User 02", "signer2@youremail.ca");
                //invitaions.Add(invitaion2);

            doc1.invitations = invitaions;

            // add tags(signature)
            var tags = new List<Tag>();
            var tag1 = new Tag();
            tag1.setTag(Code.TAG_SIGN, "sign_host", 2, 1, 132, 99, true);
            tags.Add(tag1);

            var tag2 = new Tag();
            tag2.setTag(Code.TAG_SIGN, "sign_signer1", 2, 1, 129, 173, true, "", 1);
            tags.Add(tag2);

            var tag3 = new Tag();
            tag3.setTag(Code.TAG_SIGN, "sign_signer2", 2, 1, 130, 246, true, "", 2);
            tags.Add(tag3);

            doc1.tags = tags;

            // add files <example files are in the directory of docs>
            string root = Directory.GetParent(Directory.GetCurrentDirectory()).Parent.FullName + "\\docs";
            string[] files = { root + "\\doc01.doc", root + "\\example_doc_01.pdf" };

            var doc_new = client.sendDocument(doc1, files);
            if (doc_new == null || doc_new.document == null)
            {
                Console.WriteLine("Document create failed!");
                return;
            } else
            {
                String guid_signer_01 = doc_new.document.invitations[0].GUID;
                Console.WriteLine("New document has been added, id:" + doc_new.document.id + ", Title:" + doc_new.document.title + "; GUID: " + doc_new.document.GUID + ", first signer: " + guid_signer_01);
        
                //5. pop up the document for first recipient.
                string url = UIUrl + "/documentDesigner2.html?iid=" + guid_signer_01 + "&lang=en";
                System.Diagnostics.Process.Start(url);
            }

            // 6. download URL (you can check the document's status<document.signingStatus>  53 means the signing is finished )
            String download_url = baseUrl + "v1/documents/" + doc_new.document.id + "/download?content=esign";
            Console.WriteLine(download_url);

            // 7.  if send the document Signority as a draft, need to luanch Signority page
            //if (doc1.signingStatus == Code.SIGNING_STATUS_DRAFT)
            //{
            //    string url = baseUrl + "v2/user/login?email=" + HttpUtility.UrlEncode(username) + "&password=" + HttpUtility.UrlEncode(password) +
            //        "&referrer=" + UIUrl + "documentDesigner2.html?id=" + doc_new.document.id;
            //    System.Diagnostics.Process.Start(url);
            //}

            // 8.  add file to exist document 
            //var files = client.uploadFiles(doc_id, files);

            // 9. update a document
            //docDetail = client.GetOneDocument(doc_id);
            //int page_id         = docDetail.document.files[docDetail.document.files.Count - 1].pages[0].id;
            //int invitation_id = (int)docDetail.document.invitations[0].id;
            //docDetail.document.signingStatus = 52;
            //var new_doc2 = client.updateDocument(docDetail.document);

            Console.WriteLine("Press ENTER to exit.");
            Console.Read(); // Keeps the output window open
        }


    }
}
