﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Web;
using RestSharp;

namespace SignorityAPI
{
    static class ProgramWithoutSignerTag
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {

            // local server key 
            //var apikey = "eeb38156-9706-455f-81f9-5a2b57169f1d";
            //var baseUrl = "http://localhost/api/";

            /*  test server & key
             *  login url      : http://test.signority.com/UI/
             *  user     : user01@signority.com
             *  password : signority
             *  apikey   : cf1b5433-f6da-4195-972e-b9c81fdcd3b7
             */
            //var apikey = "cf1b5433-f6da-4195-972e-b9c81fdcd3b7";
            var baseUrl = "https://test.signority.com/api/";
            var UIUrl   = "https://test.signority.com/UI/";
            var username = "user01@signority.com";
            var password = "signority";

            //var client = new Signority(apikey, "");
            // or use user name + password
            var client = new Signority(username, password);
            client.baseUrl = baseUrl;

            Console.WriteLine("Signority API Demo ...");
            // 1. get account
            var account = client.GetAccount();
            Console.WriteLine("Your account:" + account.email + "(" + account.firstname + " " + account.lastname + ")" + 
                    ", account id:" + account.id + 
                    ", status:" + account.status);

            // 2. get doc list
            var doc_list = client.GetDocumentList(1, 5);
            Console.WriteLine("");
            Console.WriteLine("You have " + doc_list.documentCount + " documents.");
            Console.WriteLine("");
            foreach (Documents doc in doc_list.documents)
            {
                Console.WriteLine("DocId: " + doc.id + ", " + doc.title + ", " + doc.GUID + "," + doc.signingStatus);
            }

            // 3. get a doc
            int doc_id = doc_list.documents[0].id;

            var docDetail = client.GetOneDocument(doc_id);
            Console.WriteLine("");
            Console.WriteLine("the detail of the first doc:" + doc_id);
            Console.WriteLine("Invitations:" + docDetail.document.invitations.Count);
            foreach (Invitaion i in docDetail.document.invitations)
            {
                Console.WriteLine("id:" + i.id + ", recipient:" + i.recipientName + "," + i.recipientEmail + "; GUID: " + i.GUID);
            }

            // 4. send a doc to signer
            var doc1 = new DocumentWithoutSignerTag();
            doc1.title = "CSharp API Test " + DateTime.Now;
            doc1.senderName = "C# API";
            doc1.senderEmail = "sender@email";
            doc1.documentType = Code.DOCUMENT_TYPE_DOCUMENT;
            doc1.signingStatus = Code.SIGNING_STATUS_DRAFT; // Code.SIGNING_STATUS_IN_PROGRESS;

            // [optional] set callback url 
            var docSettings = new DocumentSetting();
            docSettings.postBackUrl = "[http://your_callback_url]";
            doc1.documentSetting = docSettings;

            // add files <example files are in the directory of docs>
            string root = Directory.GetParent(Directory.GetCurrentDirectory()).Parent.Parent.FullName + "\\docs";
            string[] files = { root + "\\doc01.doc", root + "\\example_doc_01.pdf" };

            var doc_new = client.sendDocumentWithoutSignerTag(doc1, files);
            Console.WriteLine("New document has been added, id:" + doc_new.document.id + ", Title:" + doc_new.document.title + "; GUID: " + doc_new.document.GUID);

            //  if send the document Signority as a draft, need to luanch Signority page
            if (doc1.signingStatus == Code.SIGNING_STATUS_DRAFT)
            {
                string url = baseUrl + "v2/user/login?email=" + HttpUtility.UrlEncode(username) + "&password=" + HttpUtility.UrlEncode(password) +
                    "&referrer=" + UIUrl + "documentDesigner2.html?id=" + doc_new.document.id;
                System.Diagnostics.Process.Start(url);
            }

            // 5.  add file to exist document 
            //var files = client.uploadFiles(doc_id, files);

            // 6. update a document
            //docDetail = client.GetOneDocument(doc_id);
            //int page_id         = docDetail.document.files[docDetail.document.files.Count - 1].pages[0].id;
            //int invitation_id = (int)docDetail.document.invitations[0].id;
            //docDetail.document.signingStatus = 52;
            //var new_doc2 = client.updateDocument(docDetail.document);

            Console.WriteLine("Press ENTER to exit.");
            Console.Read(); // Keeps the output window open
        }


    }
}
