﻿using System;
using System.Collections.Generic;

using System.Net;

using RestSharp;
using Newtonsoft.Json;

namespace SignorityAPI
{

    public class Signority
    {
        private String username = "";
        private String password = "";
        private string apikey;
        static String baseUrl = "http://localhost/api/v1/";
        private String urlLogin = baseUrl + "user/login?email=securedev@signority.com&password=1230984567";

        //private readonly HttpClient client = new HttpClient();

        string responseBody = "";

        public Dictionary<string, string> AdditionalParameters { get; set; } = new Dictionary<string, string>();

        private RestClient client;
        private RestSharp.Deserializers.JsonDeserializer deserializer;
        public List<Warning> Warnings { get; private set; }
        public string Version { get; private set; }

        public Signority()
        {
            client = new RestClient();
            client.BaseUrl = new Uri(baseUrl);

            deserializer = new RestSharp.Deserializers.JsonDeserializer();
            Warnings = new List<Warning>();
        }

        public Signority(string apiKey) : this()
        {
            this.apikey = apikey;
            client.Authenticator = new RestSharp.Authenticators.HttpBasicAuthenticator(apiKey, "");
        }

        //async Task sendRequest(String url)
        //{
        //    try
        //    {
        //        HttpResponseMessage response = await client.GetAsync(url);

        //        response.EnsureSuccessStatusCode();
        //        if (response.StatusCode != HttpStatusCode.OK)
        //        {
        //            Console.WriteLine("request failed!");
        //            return false;
        //        }
        //        responseBody = await response.Content.ReadAsStringAsync();
        //        Console.WriteLine(responseBody);
        //        return true;
        //    }
        //    catch (HttpRequestException e)
        //    {
        //        Console.WriteLine("\nException Caught!");
        //        Console.WriteLine("Message :{0} ", e.Message);
        //        return "";
        //    }

        //}

        private void HandleErrors(IRestResponse response)
        {
            // If there was an exception getting the response
            if (response.ErrorException != null)
            {
                const string message = "Error retrieving response.  Check inner details for more info.";
                throw new ApplicationException(message, response.ErrorException);
            }

            // Check for errors/warnings from the response
            if (response.ContentType == "application/json")
            {
                // Check for an error
                deserializer.RootElement = "error";
                var error = deserializer.Deserialize<Error>(response);
                if (error.ErrorName != null)
                {
                    switch (error.ErrorName)
                    {
                        case "bad_request":
                            throw new BadRequestException(error.ErrorMsg, error.ErrorName);
                        case "unauthorized":
                            throw new UnauthorizedException(error.ErrorMsg, error.ErrorName);
                        case "payment_required":
                            throw new PaymentRequiredException(error.ErrorMsg, error.ErrorName);
                        case "forbidden":
                            throw new ForbiddenException(error.ErrorMsg, error.ErrorName);
                        case "not_found":
                            throw new NotFoundException(error.ErrorMsg, error.ErrorName);
                        case "conflict":
                            throw new ConflictException(error.ErrorMsg, error.ErrorName);
                        case "team_invite_failed":
                            throw new ForbiddenException(error.ErrorMsg, error.ErrorName);
                        case "invalid_recipient":
                            throw new BadRequestException(error.ErrorMsg, error.ErrorName);
                        case "signature_request_cancel_failed":
                            throw new BadRequestException(error.ErrorMsg, error.ErrorName);
                        case "maintenance":
                            throw new ServiceUnavailableException(error.ErrorMsg, error.ErrorName);
                        case "deleted":
                            throw new GoneException(error.ErrorMsg, error.ErrorName);
                        case "unknown":
                            throw new UnknownException(error.ErrorMsg, error.ErrorName);
                        case "method_not_supported":
                            throw new MethodNotAllowedException(error.ErrorMsg, error.ErrorName);
                        case "signature_request_invalid":
                            throw new ErrorException(error.ErrorMsg, error.ErrorName);
                        case "template_error":
                            throw new ErrorException(error.ErrorMsg, error.ErrorName);
                        case "invalid_reminder":
                            throw new BadRequestException(error.ErrorMsg, error.ErrorName);
                        case "exceeded_rate":
                            throw new ExceededRateLimitException(error.ErrorMsg, error.ErrorName);
                        default:
                            throw new ErrorException(error.ErrorMsg, error.ErrorName);
                    }
                }

                // Look for warnings
                deserializer.RootElement = "warnings";
                var warnings = deserializer.Deserialize<List<Warning>>(response);
                if (warnings[0].WarningName != null)
                {
                    Warnings.AddRange(warnings);
                }
            }

            // Throw exceptions appropriate for the HTTP status code
            switch (response.StatusCode)
            {
                case HttpStatusCode.BadRequest:
                    throw new BadRequestException();
                case HttpStatusCode.Unauthorized:
                    throw new UnauthorizedException();
                case HttpStatusCode.PaymentRequired:
                    throw new PaymentRequiredException();
                case HttpStatusCode.Forbidden:
                    throw new ForbiddenException();
                case HttpStatusCode.NotFound:
                    throw new NotFoundException();
                case HttpStatusCode.MethodNotAllowed:
                    throw new MethodNotAllowedException();
                case HttpStatusCode.Conflict:
                    throw new ConflictException();
                case HttpStatusCode.Gone:
                    throw new GoneException();
                case HttpStatusCode.InternalServerError:
                    throw new UnknownException();
            }

            // Throw an exception for any non-2xx status code we didn't cover above
            var statusCode = (int)response.StatusCode;
            if ((statusCode < 200) || (statusCode >= 300))
            {
                throw new ApplicationException("Received status " + statusCode + " from server. Full response:\n" + response.Content);
            }
        }

        private void InjectAdditionalParameters(RestRequest request)
        {
            foreach (KeyValuePair<string, string> entry in this.AdditionalParameters)
            {
                request.AddParameter(entry.Key, entry.Value);
            }
        }


        private T Execute<T>(RestRequest request) where T : new()
        {
            InjectAdditionalParameters(request);
            var response = client.Execute<T>(request);
            HandleErrors(response);
            return response.Data;
        }

        public Client GetAccount()
        {
            var request = new RestRequest("client");
            request.RootElement = "client";
            return Execute<Client>(request);
        }

        private void RequireAuthentication()
        {
            if (client.Authenticator == null)
            {
                throw new UnauthorizedAccessException("This method requires authentication");
            }
        }


    }

}

